﻿using System;
using System.Linq;
using System.Threading;

namespace DelegatesTrying
{
    internal delegate int BinaryOperation(int a, int b);

    class Program
    {
       
        static void Main(string[] args)
        {
            //BinaryOperations();
            //ClosureExample();
            //LambdaExample();
        }

        private static void LambdaExample()
        {
            int[] numbers = new[] {2, 4, 5, 4, 23};
            var evenNumbers = numbers.Where(delegate(int i) { return i%2 == 0; });
            var oddNumbers = numbers.Where(i => i%2 == 1);
            Console.WriteLine(String.Join(", ", evenNumbers));
            Console.WriteLine(String.Join(", ", oddNumbers));
        }

        private static void ClosureExample()
        {
            for (int i = 0; i < 10; i++)
            { 
                // 'i' is in the closure object of the delegate.
                // The value of i changes
                new Thread(delegate() { Console.Write(i + " "); }).Start();
            }
            Console.ReadLine();

            // Works in C# 5.0
            // C# 4.5 same number migh show up more than once
            foreach (int i in Enumerable.Range(0, 10))
            {
                new Thread(() => Console.Write(i + " ")).Start();
            }

            Console.ReadLine();
            for (int i = 0; i < 10; i++)
            {
                int i1 = i; // local variable. Always works: no duplicates
                new Thread(() => Console.Write(i1 + " ")).Start();
            }
        }

        public static int Plus(int a, int b)
        {
            Console.WriteLine("Plus called");
            return a + b;
        }

        public static int Minus(int a, int b)
        {
            Console.WriteLine("Minus called");
            return a - b;
        }

        private static void BinaryOperations()
        {
            BinaryOperation operation = Plus;
            //operation += Minus;
            Console.WriteLine(operation(3, 2));
            var op = new Func<int, int, int>(Plus);
            int res = op.Invoke(3, 2);
            Console.WriteLine(res);
        }
    }
}
